@@ -38,7 +38,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.19 2004/01/24 09:56:43 marco Exp $
+ * $Id: MozDownload.cpp,v 1.31.4.6 2004/05/19 15:11:32 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +49,9 @@
 #include "mozilla-download.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-file-helpers.h"
+#include "ephy-debug.h"
+
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
 
@@ -57,40 +60,60 @@
 #include "nsDirectoryServiceUtils.h"
 #include "nsIRequest.h"
 #include "nsIMIMEInfo.h"
+#include "nsIFileURL.h"
 #include "netCore.h"
+#include "nsNetUtil.h"
 
 const char* const persistContractID = "@mozilla.org/embedding/browser/nsWebBrowserPersist;1";
 
 MozDownload::MozDownload() :
-	mGotFirstStateChange(false), mIsNetworkTransfer(false),
+	mMaxSize(-1),
 	mStatus(NS_OK),
 	mEmbedPersist(nsnull),
 	mDownloadState(EPHY_DOWNLOAD_DOWNLOADING)
 {
+	LOG ("MozDownload ctor (%p)", (void *) this)
 }
 
 MozDownload::~MozDownload()
 {
+	LOG ("MozDownload dtor (%p)", (void *) this)
+
+	NS_ASSERTION (!mEphyDownload, "MozillaDownload still alive!");
 }
 
+#if MOZILLA_SNAPSHOT < 16
 NS_IMPL_ISUPPORTS2(MozDownload, nsIDownload, nsIWebProgressListener)
+#else
+NS_IMPL_ISUPPORTS3(MozDownload, nsIDownload, nsITransfer, nsIWebProgressListener)
+#endif
 
 NS_IMETHODIMP
+#if MOZILLA_SNAPSHOT < 16
 MozDownload::InitForEmbed (nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisplayName,
+#else
+MozDownload::InitForEmbed (nsIURI *aSource, nsIURI *aTarget, const PRUnichar *aDisplayName,
+#endif
 		           nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist,
-		           MozillaEmbedPersist *aEmbedPersist)
+		           MozillaEmbedPersist *aEmbedPersist, PRInt32 aMaxSize)
 {
 	mEmbedPersist = aEmbedPersist;
+	mMaxSize = aMaxSize;
 	return Init (aSource, aTarget, aDisplayName, aMIMEInfo, startTime, aPersist);
 }
 
-/* void init (in nsIURI aSource, in nsILocalFile aTarget, in wstring aDisplayName, in nsIMIMEInfo aMIMEInfo, in long long startTime, in nsIWebBrowserPersist aPersist); */
+/* void init (in nsIURI aSource, in nsIURI aTarget, in wstring aDisplayName, in nsIMIMEInfo aMIMEInfo, in long long startTime, in nsIWebBrowserPersist aPersist); */
+#if MOZILLA_SNAPSHOT < 16
 NS_IMETHODIMP
 MozDownload::Init(nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisplayName,
-		   nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist)
+		  nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist)
+#else
+NS_IMETHODIMP
+MozDownload::Init(nsIURI *aSource, nsIURI *aTarget, const PRUnichar *aDisplayName,
+		  nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist)
+#endif
 {
 	PRBool addToView = PR_TRUE;
-	nsresult rv;
 
 	if (mEmbedPersist)
 	{
@@ -107,7 +130,7 @@ MozDownload::Init(nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisp
 	mTotalProgress = 0;
 	mCurrentProgress = 0;
 	mPercentComplete = 0;
-	mInterval = 4000; /* in ms */
+	mInterval = 200000; /* microsec */
 	mLastUpdate = mStartTime;
 	mMIMEInfo = aMIMEInfo;
 
@@ -122,9 +145,11 @@ MozDownload::Init(nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisp
 		DownloaderView *dview;
 		dview = EPHY_DOWNLOADER_VIEW
 			(ephy_embed_shell_get_downloader_view (embed_shell));
-		mEphyDownload = mozilla_download_new ();
-		MOZILLA_DOWNLOAD (mEphyDownload)->moz_download = this;
+		mEphyDownload = mozilla_download_new (this);
+		g_object_add_weak_pointer (G_OBJECT (mEphyDownload),
+					   (gpointer *) &mEphyDownload);
 		downloader_view_add_download (dview, mEphyDownload);
+		g_object_unref (mEphyDownload);
 	}
 	else
 	{
@@ -143,8 +168,13 @@ MozDownload::GetSource(nsIURI **aSource)
 	return NS_OK;
 }
 
+#if MOZILLA_SNAPSHOT < 16
 NS_IMETHODIMP
 MozDownload::GetTarget(nsILocalFile **aTarget)
+#else
+NS_IMETHODIMP
+MozDownload::GetTarget(nsIURI **aTarget)
+#endif
 {
 	NS_ENSURE_ARG_POINTER(aTarget);
 	NS_IF_ADDREF(*aTarget = mDestination);
@@ -152,6 +182,23 @@ MozDownload::GetTarget(nsILocalFile **aTarget)
 	return NS_OK;
 }
 
+#if MOZILLA_SNAPSHOT > 15
+NS_IMETHODIMP
+MozDownload::GetTargetFile (nsILocalFile** aTargetFile)
+{
+	nsresult rv;
+                                                                                                                              
+	nsCOMPtr<nsIFileURL> fileURL = do_QueryInterface(mDestination, &rv);
+	if (NS_FAILED(rv)) return rv;
+                                                                                                                              
+	nsCOMPtr<nsIFile> file;
+	rv = fileURL->GetFile(getter_AddRefs(file));
+	if (NS_SUCCEEDED(rv))
+		rv = CallQueryInterface(file, aTargetFile);
+	return rv;
+}
+#endif
+
 NS_IMETHODIMP
 MozDownload::GetPersist(nsIWebBrowserPersist **aPersist)
 {
@@ -268,19 +315,11 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 {
 	nsresult rv;
 
-	/* For a file download via the external helper app service, we will never get a start
-           notification. The helper app service has gotten that notification before it created us. */
-	if (!mGotFirstStateChange)
-	{
-        	mIsNetworkTransfer = ((aStateFlags & STATE_IS_NETWORK) != 0);
-	        mGotFirstStateChange = PR_TRUE;
-	}
-
 	if (NS_FAILED(aStatus) && NS_SUCCEEDED(mStatus))
         	mStatus = aStatus;
-  
+
 	/* We will get this even in the event of a cancel */
-	if ((aStateFlags & STATE_STOP) && (!mIsNetworkTransfer || (aStateFlags & STATE_IS_NETWORK)))
+	if (aStateFlags & STATE_STOP)
 	{
 		/* Keep us alive */
 		nsCOMPtr<nsIDownload> kungFuDeathGrip(this);
@@ -311,8 +350,8 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 		else if (NS_SUCCEEDED (aStatus))
 		{
 			GnomeVFSMimeApplication *helperApp;
-			char *mimeType;	
-
+#if MOZILLA_SNAPSHOT < 18
+			char *mimeType;
 			rv = mMIMEInfo->GetMIMEType (&mimeType);
 			NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
@@ -325,13 +364,31 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 			   if we have to open the saved file */
 			if ((strcmp (NS_ConvertUCS2toUTF8 (description).get(), "gnome-default") == 0) &&
 			    helperApp)
+#else
+			nsCAutoString mimeType;
+			rv = mMIMEInfo->GetMIMEType (mimeType);
+			NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+			helperApp = gnome_vfs_mime_get_default_application (mimeType.get()); 
+
+			nsAutoString description;
+			mMIMEInfo->GetApplicationDescription (description);
+
+			/* HACK we use the application description to decide
+			   if we have to open the saved file */
+			if ((strcmp (NS_ConvertUCS2toUTF8 (description).get(), "gnome-default") == 0) &&
+			    helperApp)
+#endif
 			{
 				GList *params = NULL;
 				char *param;
 				nsCAutoString aDest;
 
+#if MOZILLA_SNAPSHOT < 16
 				mDestination->GetNativePath (aDest);
-
+#else
+				mDestination->GetSpec (aDest);
+#endif
 				param = gnome_vfs_make_uri_canonical (aDest.get ());
 				params = g_list_append (params, param);
 				gnome_vfs_mime_application_launch (helperApp, params);
@@ -339,8 +396,10 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 
 				g_list_free (params);
 			}
-
+#if MOZILLA_SNAPSHOT < 16
 			nsMemory::Free (mimeType);
+			nsMemory::Free (description);
+#endif
 			gnome_vfs_mime_application_free (helperApp);
 		}
 	}
@@ -353,6 +412,13 @@ MozDownload::OnProgressChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest
 			      PRInt32 aCurSelfProgress, PRInt32 aMaxSelfProgress,
 			      PRInt32 aCurTotalProgress, PRInt32 aMaxTotalProgress)
 {
+	if (mMaxSize >= 0 &&
+	    ((aMaxTotalProgress > 0 && mMaxSize < aMaxTotalProgress) ||
+	     mMaxSize < aCurTotalProgress))
+	{
+		Cancel ();
+	}
+
 	if (!mRequest)
 		mRequest = aRequest;
 
@@ -447,14 +513,15 @@ MozDownload::Resume()
 nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI,
 				  nsILocalFile* inDestFile, const char *contentType,
 				  nsIURI* inOriginalURI, MozillaEmbedPersist *embedPersist,
-				  nsIInputStream *postData, nsISupports *aCacheKey)
+				  nsIInputStream *postData, nsISupports *aCacheKey,
+				  PRInt32 aMaxSize)
 {
 	nsresult rv = NS_OK;
 
 	EmbedPersistFlags ephy_flags;
 	ephy_flags = ephy_embed_persist_get_flags (EPHY_EMBED_PERSIST (embedPersist));
 
-	PRBool isHTML = (domDocument && contentType &&
+	PRBool isHTML = (contentType &&
 			(strcmp (contentType, "text/html") == 0 ||
 			 strcmp (contentType, "text/xml") == 0 ||
 			 strcmp (contentType, "application/xhtml+xml") == 0));
@@ -467,67 +534,65 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 	nsAutoString fileDisplayName;
 	inDestFile->GetLeafName(fileDisplayName);
 
+	#if MOZILLA_SNAPSHOT >= 16
+	nsCOMPtr<nsIURI> destURI;
+	NS_NewFileURI (getter_AddRefs(destURI), inDestFile);
+	#endif
+
 	MozDownload *downloader = new MozDownload ();
 	/* dlListener attaches to its progress dialog here, which gains ownership */
+	#if MOZILLA_SNAPSHOT < 16
 	rv = downloader->InitForEmbed (inOriginalURI, inDestFile, fileDisplayName.get(),
-				       nsnull, timeNow, webPersist, embedPersist);
+	#else
+	rv = downloader->InitForEmbed (inOriginalURI, destURI, fileDisplayName.get(),
+	#endif
+				       nsnull, timeNow, webPersist, embedPersist, aMaxSize);
 	NS_ENSURE_SUCCESS (rv, rv);
 
 	PRInt32 flags = nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
+
+	if (!domDocument && !isHTML && !(ephy_flags & EMBED_PERSIST_COPY_PAGE))
+	{
+		flags |= nsIWebBrowserPersist::PERSIST_FLAGS_NO_CONVERSION;
+	}
 	if (ephy_flags & EMBED_PERSIST_COPY_PAGE)
 	{
 		flags |= nsIWebBrowserPersist::PERSIST_FLAGS_FROM_CACHE;
 	}
 	webPersist->SetPersistFlags(flags);
 
-	if (!isHTML || ephy_flags & EMBED_PERSIST_COPY_PAGE)
+	if (!domDocument || !isHTML || ephy_flags & EMBED_PERSIST_COPY_PAGE)
 	{
 		rv = webPersist->SaveURI (sourceURI, aCacheKey, nsnull,
 					  postData, nsnull, inDestFile);
 	}
 	else
 	{
-		if (!domDocument) return rv;  /* should never happen */
-    
 		PRInt32 encodingFlags = 0;
 		nsCOMPtr<nsILocalFile> filesFolder;
 
-    		if (contentType && strcmp (contentType, "text/plain") == 0)
-		{
-			/* Create a local directory in the same dir as our file.  It
-			   will hold our associated files. */
+		/**
+		 * Construct a directory path to hold the associated files; mozilla
+		 * will create the directory as needed.
+		 */
 
-			filesFolder = do_CreateInstance("@mozilla.org/file/local;1");
-			nsAutoString unicodePath;
-			inDestFile->GetPath(unicodePath);
-			filesFolder->InitWithPath(unicodePath);
+		filesFolder = do_CreateInstance("@mozilla.org/file/local;1");
+		nsAutoString unicodePath;
+		inDestFile->GetPath(unicodePath);
+		filesFolder->InitWithPath(unicodePath);
       
-			nsAutoString leafName;
-			filesFolder->GetLeafName(leafName);
-			nsAutoString nameMinusExt(leafName);
-			PRInt32 index = nameMinusExt.RFind(".");
-			if (index >= 0)
-			{
-				nameMinusExt.Left(nameMinusExt, index);
-			}
-
-			nameMinusExt += NS_LITERAL_STRING(" Files");
-			filesFolder->SetLeafName(nameMinusExt);
-			PRBool exists = PR_FALSE;
-			filesFolder->Exists(&exists);
-			if (!exists)
-			{
-				rv = filesFolder->Create(nsILocalFile::DIRECTORY_TYPE, 0755);
-				if (NS_FAILED(rv)) return rv;
-			}
-		}
-		else
+		nsAutoString leafName;
+		filesFolder->GetLeafName(leafName);
+		nsAutoString nameMinusExt(leafName);
+		PRInt32 index = nameMinusExt.RFind(".");
+		if (index >= 0)
 		{
-			encodingFlags |= nsIWebBrowserPersist::ENCODE_FLAGS_FORMATTED |
-					 nsIWebBrowserPersist::ENCODE_FLAGS_ABSOLUTE_LINKS |
-					 nsIWebBrowserPersist::ENCODE_FLAGS_NOFRAMES_CONTENT;
+			nameMinusExt.Left(nameMinusExt, index);
 		}
 
+		nameMinusExt += NS_LITERAL_STRING (" Files");
+		filesFolder->SetLeafName(nameMinusExt);
+
 		rv = webPersist->SaveDocument (domDocument, inDestFile, filesFolder,
 					       contentType, encodingFlags, 80);
 	}
@@ -538,47 +603,85 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 static char*
 GetFilePath (const char *filename)
 {
-	char *path, *download_dir, *expanded, *tmp;
+	char *path = NULL;
+	char *download_dir, *converted_dp, *expanded;
 
 	download_dir = eel_gconf_get_string (CONF_STATE_DOWNLOAD_DIR);
 
 	if (!download_dir)
 	{
 		/* Emergency download destination */
-		download_dir = g_strdup (g_get_home_dir ());
+		return g_build_filename (g_get_home_dir (), filename, NULL);
 	}
-	else if (g_utf8_collate (download_dir, "Downloads") == 0)
+
+	if (g_utf8_collate (download_dir, "Downloads") == 0)
 	{
-		tmp = g_build_filename (g_get_home_dir (), "Desktop",
-				        _("Downloads"), NULL);
+		const char *default_dir;
+
 		g_free (download_dir);
-		download_dir = tmp;
+
+		default_dir = ephy_file_downloads_dir ();
+		ephy_ensure_dir_exists (default_dir);
+
+		return g_build_filename (default_dir, filename, NULL);
 	}
 
-	expanded = gnome_vfs_expand_initial_tilde (download_dir);
-	path = g_build_filename (expanded, filename, NULL);
-	g_free (expanded);
-
+	converted_dp = g_filename_from_utf8 (download_dir, -1, NULL, NULL, NULL);
 	g_free (download_dir);
 
+	if (converted_dp)
+	{
+		expanded = gnome_vfs_expand_initial_tilde (converted_dp);
+		ephy_ensure_dir_exists (expanded);
+		path = g_build_filename (expanded, filename, NULL);
+
+		g_free (expanded);
+		g_free (converted_dp);
+	}
+	else
+	{
+		/* Fallback, see FIXME above too */
+		path = g_build_filename (g_get_home_dir (), filename, NULL);
+	}
+
 	return path;
 }
 
 nsresult BuildDownloadPath (const char *defaultFileName, nsILocalFile **_retval)
 {
 	char *path;
-	int i = 0;
 
 	path = GetFilePath (defaultFileName);
 	
-	while (g_file_test (path, G_FILE_TEST_EXISTS))
+	if (g_file_test (path, G_FILE_TEST_EXISTS))
 	{
-		g_free (path);
+		int i = 1;
+		char *dot_pos, *serial = NULL;
+		GString *tmp_path;
+		gssize position;
 
-		char *tmp_path;
-		tmp_path = g_strdup_printf ("%s.%d", defaultFileName,  ++i);
-		path = GetFilePath (tmp_path);
-		g_free (tmp_path);
+		dot_pos = g_strrstr (defaultFileName, ".");
+		if (dot_pos)
+		{
+			position = dot_pos - defaultFileName;
+		}
+		else
+		{
+			position = strlen (defaultFileName);
+		}
+		tmp_path = g_string_new (NULL);
+
+		do {
+			g_free (path);
+			g_string_assign (tmp_path, defaultFileName);
+			serial = g_strdup_printf ("(%d)", i++);
+			g_string_insert (tmp_path, position, serial);
+			g_free (serial);
+			path = GetFilePath (tmp_path->str);
+			
+		} while (g_file_test (path, G_FILE_TEST_EXISTS));
+		
+		g_string_free (tmp_path, TRUE);
 	}
 
 	nsCOMPtr <nsILocalFile> destFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
