@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-string.h,v 1.8 2003/12/13 22:29:00 chpe Exp $
  */
 
 #ifndef EPHY_STRING_H
@@ -26,13 +28,14 @@ G_BEGIN_DECLS
 gboolean	 ephy_string_to_int			(const char *string,
 							 gulong *integer);
 
-char		*ephy_string_strip_chr			(const char *source,
-							 char remove_this);
+char		*ephy_string_blank_chr			(char *source);
 
-char		*ephy_string_shorten			(const gchar *str,
-							 gint target_length);
+char		*ephy_string_shorten			(const char *str,
+							 int target_length);
 
-char		*ephy_string_elide_underscores		(const gchar *original);
+char		*ephy_string_elide_underscores		(const char *original);
+
+char            *ephy_string_double_underscores         (const char *string);
 
 G_END_DECLS
 
