@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-import.c,v 1.26.4.1 2004/03/22 09:17:33 marco Exp $
+ *  $Id: ephy-bookmarks-import.c,v 1.26.4.2 2004/06/08 21:42:26 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -105,7 +105,8 @@ ephy_bookmarks_import (EphyBookmarks *bookmarks,
 		return ephy_bookmarks_import_xbel (bookmarks, filename);
 	}
 	else if (strstr (filename, MOZILLA_BOOKMARKS_DIR) != NULL ||
-                 strstr (filename, FIREBIRD_BOOKMARKS_DIR) != NULL)
+                 strstr (filename, FIREBIRD_BOOKMARKS_DIR) != NULL ||
+		 strstr (filename, FIREFOX_BOOKMARKS_DIR) != NULL)
 	{
 		return ephy_bookmarks_import_mozilla (bookmarks, filename);
 	}
