@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-nautilus-view.c,v 1.31 2004/02/24 17:43:48 chpe Exp $
+ *  $Id: ephy-nautilus-view.c,v 1.31.4.1 2004/03/22 09:21:30 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,7 +46,7 @@ static void		gnv_embed_title_cb 			(EphyEmbed *embed,
 								 EphyNautilusView *view);
 static void		gnv_embed_new_window_cb			(EphyEmbed *embed, 
 								 EphyEmbed **new_embed,
-								 EmbedChromeMask chromemask,
+								 EphyEmbedChrome chromemask,
 								 EphyNautilusView *view);
 static void		gnv_embed_link_message_cb 		(EphyEmbed *embed, 
 								 EphyNautilusView *view);
@@ -421,7 +421,7 @@ gnv_stop_loading_cb (EphyNautilusView *view, gpointer user_data)
 
 static void
 gnv_embed_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
-			 EmbedChromeMask chromemask, EphyNautilusView *view)
+			 EphyEmbedChrome chromemask, EphyNautilusView *view)
 {
 	EphyTab *new_tab;
 	EphyWindow *window;
